@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2008 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp.cc,v 1.79 2007/08/16 15:39:52 lav Exp $ */
+/* $Id: lftp.cc,v 1.83 2009/07/21 08:41:12 lav Exp $ */
 
 #include <config.h>
 
@@ -51,7 +51,7 @@
 #include "misc.h"
 #include "ArgV.h"
 
-#include "confpaths.h"
+#include "configmake.h"
 
 #include "lftp_rl.h"
 #include "complete.h"
@@ -138,8 +138,7 @@ public:
 
 	 if(ctty) // controlling terminal
 	 {
-	    pid_t term_pg=tcgetpgrp(0);
-	    if(term_pg!=(pid_t)-1 && getpgrp()!=term_pg)
+	    if(!in_foreground_pgrp())
 	    {
 	       // looks like we are in background. Can't read from tty
 	       exec->Timeout(500);
@@ -264,7 +263,7 @@ CMD(history)
    if(const char *arg = args->getcurr()) {
       if(!strcasecmp(arg, "all"))
 	 cnt = -1;
-      else if(isdigit(arg[0]))
+      else if(isdigit((unsigned char)arg[0]))
 	 cnt = atoi(arg);
       else {
 	 eprintf(_("%s: %s - not a number\n"), args->a0(), args->getcurr());
@@ -308,6 +307,14 @@ static void sig_term(int sig)
 {
    time_t t=time(0);
    printf(_("[%lu] Terminated by signal %d. %s"),(unsigned long)getpid(),sig,ctime(&t));
+   if(top_exec) {
+      top_exec->KillAll();
+      alarm(30);
+      while(Job::NumberOfJobs()>0) {
+	 SMTask::Schedule();
+	 SMTask::Block();
+      }
+   }
    exit(1);
 }
 
